@@ -21,7 +21,7 @@ from .core import (
 from discord import http, ui
 from discord.ext import commands
 from discord.enums import InteractionType
-from typing import List, Optional, Tuple, Union, Dict, Mapping, Callable, Any
+from typing import List, Optional, Tuple, Union, Dict, Mapping, Callable, Any, Awaitable
 
 
 __all__ = (
@@ -31,8 +31,8 @@ __all__ = (
 
 class ApplicationMixin:
     """The mixin for appcommands module"""
-    def __init__(self, *args, **oldkwargs) -> None:
-        kwargs = oldkwargs.copy()
+    def __init__(self, *args, **kwargs) -> None:
+        oldkwargs = kwargs.copy()
 
         if not kwargs.get('command_prefix'):
             kwargs["command_prefix"] = " ".join(secrets.token_urlsafe(5000).split('_'))
@@ -66,7 +66,7 @@ class ApplicationMixin:
         self.add_listener(self.__connectlistener, "on_connect")
         self.add_listener(self.interaction_handler, "on_interaction")
 
-    def add_app_command(self, command: BaseCommand) -> None:
+    def add_app_command(self, command: BaseCommand, *, on_discord: bool = False) -> Union[None, Awaitable]:
         """Adds a app command,
         usually used when subclassed
 
@@ -74,9 +74,14 @@ class ApplicationMixin:
 
         Parameters
         ------------
-        command: :class:`~appcommands.models.BaseCommand`
-            The command which is to be added"""
+        command: :class:`~appcommands.core.BaseCommand`
+            The command which is to be added
+        on_discord: :class:`~bool`
+            Whether to register all pending commands on discord
+            needs to be awaited when passed ``True`` (default: ``False``)"""
         self.to_register.append(command)
+        if on_discord:
+            return self.register_commands()
 
     def remove_app_command(self, command: BaseCommand) -> None:
         """Remove an application command from the internal list
@@ -89,11 +94,16 @@ class ApplicationMixin:
         command: :class:`appcommands.BaseCommand`
             The command to remove.
         """
-        self.__appcommands.pop(command.id)
-        self.__subcommands.pop(command.id)
-        self.__usercommands.pop(command.id)
-        self.__slashcommands.pop(command.id)
-        self.__messagecommands.pop(command.id)
+        if command.id in self.__appcommands:
+            self.__appcommands.pop(command.id)
+        if command.id in self.__subcommands:
+            self.__subcommands.pop(command.id)
+        if command.id in self.__usercommands:
+            self.__usercommands.pop(command.id)
+        if command.id in self.__slashcommands:
+            self.__slashcommands.pop(command.id)
+        if command.id in self.__messagecommands:
+            self.__messagecommands.pop(command.id)
 
     def slashcommand(self, cls=MISSING, **kwargs) -> Callable[[Callable], SlashCommand]:
         r"""A decorator which adds a slash command to bot
@@ -263,6 +273,8 @@ class ApplicationMixin:
         commands = []
         perms = {}
         registered_commands = await self.http.get_global_commands(self.user.id)
+        self.to_register.extend([c for c in list(self.appcommands.values()) if ((isinstance(c,(SubCommandGroup,SlashCommand))) and (not bool(c.parent))) or (not isinstance(c,(SubCommandGroup,SlashCommand)))])
+
         for command in [cmd for cmd in self.to_register if not cmd.guild_ids]:
             json = command.to_dict()
             if len(registered_commands) > 0:
@@ -273,8 +285,8 @@ class ApplicationMixin:
                 ]
                 if matches:
                     json["id"] = matches[0]["id"]
-            commands.append(json)
 
+            commands.append(json)
         guild_commands = {}
         async for guild in self.fetch_guilds(limit=None):
             guild_commands[guild.id] = []
@@ -282,9 +294,16 @@ class ApplicationMixin:
 
         for command in [cmd for cmd in self.to_register if cmd.guild_ids]:
             json = command.to_dict()
-            for guild_id in command.guild_ids:
-                to_update = guild_commands[guild_id]
-                guild_commands[guild_id] = to_update + [json]
+            if command.guild_ids is ALL_GUILDS:
+                command.guild_ids = []
+                command.all_guilds = True
+                for guild_id in guild_commands:
+                    guild_commands[guild_id] = guild_commands[guild_id]+[json]
+                    command.guild_ids.append(guild_id)
+            else:
+                for guild_id in command.guild_ids:
+                    to_update = guild_commands[guild_id]
+                    guild_commands[guild_id] = to_update + [json]
 
         
         for guild_id in guild_commands:
@@ -293,7 +312,6 @@ class ApplicationMixin:
 
             try:
                 cmds = await self.http.bulk_upsert_guild_commands(self.user.id, guild_id, guild_commands[guild_id])
-                
             except Exception as e:
                 print(f"Failed to add guild commands for guild {guild_id}")
                 traceback.print_exc()
@@ -365,8 +383,8 @@ class ApplicationMixin:
 
     async def __connectlistener(self):
         if not self.__connected:
-            self.__connected = True
             await self.register_commands()
+            self.__connected = True
             self.remove_listener(self.__connectlistener, 'on_connect')
 
     @property
@@ -439,9 +457,13 @@ class ApplicationMixin:
         Mapping[:class:`~str`, Union[:class:`~appcommands.SlashCommand`, :class:`~appcommands.models.SubCommandGroup`]]
         """
         ret = {}
-
-        for id, cmd in self.__slashcommands.items():
-            ret[self.__slashcommands[id]] = cmd
+        for cmd in self.__slashcommands.values():
+            if not isinstance(cmd, SubCommandGroup): ret[cmd.full_name] = cmd
+        for cmd in self.__subcommands.values():
+            if not isinstance(cmd, dict): ret[cmd.full_name] = cmd
+            else:
+                for _cmd in cmd.values():
+                    ret[_cmd.full_name] = _cmd
 
         return types.MappingProxyType(ret)
 
@@ -457,7 +479,7 @@ class ApplicationMixin:
 
         Returns
         ---------
-        Union[:class:`~appcommands.SlashCommand`, :class:`~appcommands.SubCommandGroup`]
+        Union[:class:`~appcommands.SlashCommand`, :class:`~appcommands.SubCommandGroup`, :class:`None`]
             The found thing"""
         return (self.get_slash_commands()).get(name)
 
@@ -494,7 +516,7 @@ class ApplicationMixin:
         return (self.get_user_commands()).get(name)
 
     def get_message_commands(self) -> Mapping[str, MessageCommand]:
-        """Gets every user commands registered in the current running instance
+        """Gets every message commands registered in the current running instance
 
         .. versionadded:: 2.0
 
@@ -523,6 +545,36 @@ class ApplicationMixin:
             The found thing"""
         return (self.get_message_commands()).get(name)
 
+    def get_app_commands(self) -> Mapping[str, BaseCommand]:
+        """Gets every app commands registered in the current running instance
+
+        .. versionadded:: 2.0
+
+        Returns
+        ---------
+        Mapping[:class:`~str`, :class:`appcommands.core.BaseCommand`]
+        """
+        ret = {}
+
+        for id, cmd in self.appcommands.items():
+            ret[self.__appcommands[id].name] = cmd
+
+        return types.MappingProxyType(ret)
+
+    def get_app_command(self, name: str) -> BaseCommand:
+        """Gives a app command registered in this module
+        
+        Parameters
+        -----------
+        name: :class:`~str`
+            the name from which the message command is to be found
+
+        Returns
+        ---------
+        :class:`appcommands.core.BaseCommand`
+            The found thing"""
+        return (self.get_app_commands()).get(name)
+
     def get_interaction_context(self, interaction: discord.Interaction) -> InteractionContext:
         """The method usually implemented to use custom contexts
 
